@@ -1,13 +1,13 @@
 """Init file."""
 
-from gpt_index.data_structs.data_structs import (
+from gpt_index.data_structs.data_structs_v2 import (
     IndexDict,
     IndexGraph,
     IndexList,
     KeywordTable,
-    Node,
 )
-from gpt_index.data_structs.table import StructDatapoint
+from gpt_index.data_structs.node_v2 import Node
+from gpt_index.data_structs.table_v2 import StructDatapoint
 
 __all__ = [
     "Node",
