@@ -4,9 +4,10 @@ import logging
 from abc import abstractmethod
 from typing import Any, Callable, Dict, List, Optional, Sequence, cast
 
+from gpt_index.data_structs.node_v2 import Node
 from gpt_index.data_structs.table import StructDatapoint
-from gpt_index.indices.prompt_helper import PromptHelper
 from gpt_index.indices.response.builder import ResponseBuilder, TextChunk
+from gpt_index.indices.service_context import ServiceContext
 from gpt_index.langchain_helpers.chain_wrapper import LLMPredictor
 from gpt_index.langchain_helpers.sql_wrapper import SQLDatabase
 from gpt_index.langchain_helpers.text_splitter import TextSplitter
@@ -50,8 +51,7 @@ class SQLDocumentContextBuilder:
     def __init__(
         self,
         sql_database: SQLDatabase,
-        llm_predictor: Optional[LLMPredictor] = None,
-        prompt_helper: Optional[PromptHelper] = None,
+        service_context: Optional[ServiceContext] = None,
         text_splitter: Optional[TextSplitter] = None,
         table_context_prompt: Optional[TableContextPrompt] = None,
         refine_table_context_prompt: Optional[RefineTableContextPrompt] = None,
@@ -62,11 +62,8 @@ class SQLDocumentContextBuilder:
         if sql_database is None:
             raise ValueError("sql_database must be provided.")
         self._sql_database = sql_database
-        self._llm_predictor = llm_predictor or LLMPredictor()
-        self._prompt_helper = prompt_helper or PromptHelper.from_llm_predictor(
-            self._llm_predictor
-        )
         self._text_splitter = text_splitter
+        self._service_context = service_context or ServiceContext.from_defaults()
         self._table_context_prompt = (
             table_context_prompt or DEFAULT_TABLE_CONTEXT_PROMPT
         )
@@ -102,12 +99,13 @@ class SQLDocumentContextBuilder:
         )
         text_splitter = (
             self._text_splitter
-            or self._prompt_helper.get_text_splitter_given_prompt(prompt_with_schema, 1)
+            or self._service_context.prompt_helper.get_text_splitter_given_prompt(
+                prompt_with_schema, 1
+            )
         )
         # we use the ResponseBuilder to iteratively go through all texts
         response_builder = ResponseBuilder(
-            self._prompt_helper,
-            self._llm_predictor,
+            self._service_context,
             prompt_with_schema,
             refine_prompt_with_schema,
         )
@@ -130,13 +128,11 @@ class BaseStructDatapointExtractor:
     def __init__(
         self,
         llm_predictor: LLMPredictor,
-        text_splitter: TextSplitter,
         schema_extract_prompt: SchemaExtractPrompt,
         output_parser: OUTPUT_PARSER_TYPE,
     ) -> None:
         """Initialize params."""
         self._llm_predictor = llm_predictor
-        self._text_splitter = text_splitter
         self._schema_extract_prompt = schema_extract_prompt
         self._output_parser = output_parser
 
@@ -180,9 +176,9 @@ class BaseStructDatapointExtractor:
     def _get_schema_text(self) -> str:
         """Get schema text for extracting relevant info from unstructured text."""
 
-    def insert_datapoint_from_document(self, document: BaseDocument) -> None:
+    def insert_datapoint_from_nodes(self, nodes: Sequence[Node]) -> None:
         """Extract datapoint from a document and insert it."""
-        text_chunks = self._text_splitter.split_text(document.get_text())
+        text_chunks = [node.get_text() for node in nodes]
         fields = {}
         for i, text_chunk in enumerate(text_chunks):
             fmt_text_chunk = truncate_text(text_chunk, 50)
@@ -200,7 +196,6 @@ class BaseStructDatapointExtractor:
             # validate fields with col_types_map
             new_cur_fields = self._clean_and_validate_fields(cur_fields)
             fields.update(new_cur_fields)
-
         struct_datapoint = StructDatapoint(fields)
         if struct_datapoint is not None:
             self._insert_datapoint(struct_datapoint)
