@@ -6,15 +6,12 @@ import logging
 from typing import Dict, List, Optional, Sequence, Tuple
 
 from gpt_index.async_utils import run_async_tasks
-from gpt_index.data_structs.data_structs import IndexGraph, Node
-from gpt_index.indices.node_utils import get_text_splits_from_document
-from gpt_index.indices.prompt_helper import PromptHelper
+from gpt_index.data_structs.data_structs_v2 import IndexGraph
+from gpt_index.data_structs.node_v2 import Node
+from gpt_index.docstore_v2 import DocumentStore
+from gpt_index.indices.service_context import ServiceContext
 from gpt_index.indices.utils import get_sorted_node_list, truncate_text
-from gpt_index.langchain_helpers.chain_wrapper import LLMPredictor
-from gpt_index.langchain_helpers.text_splitter import TextSplitter
-from gpt_index.logger.base import LlamaLogger
 from gpt_index.prompts.prompts import SummaryPrompt
-from gpt_index.schema import BaseDocument
 
 logger = logging.getLogger(__name__)
 
@@ -31,47 +28,27 @@ class GPTTreeIndexBuilder:
         self,
         num_children: int,
         summary_prompt: SummaryPrompt,
-        llm_predictor: LLMPredictor,
-        prompt_helper: PromptHelper,
-        text_splitter: TextSplitter,
+        service_context: ServiceContext,
+        docstore: Optional[DocumentStore] = None,
         use_async: bool = False,
-        llama_logger: Optional[LlamaLogger] = None,
     ) -> None:
         """Initialize with params."""
         if num_children < 2:
             raise ValueError("Invalid number of children.")
         self.num_children = num_children
         self.summary_prompt = summary_prompt
-        self._llm_predictor = llm_predictor
-        self._prompt_helper = prompt_helper
-        self._text_splitter = text_splitter
+        self._service_context = service_context
         self._use_async = use_async
-        self._llama_logger = llama_logger or LlamaLogger()
+        self._docstore = docstore or DocumentStore()
 
-    def _get_nodes_from_document(
-        self, start_idx: int, document: BaseDocument
-    ) -> Dict[int, Node]:
-        """Add document to index."""
-        # NOTE: summary prompt does not need to be partially formatted
-        text_splits = get_text_splits_from_document(
-            document=document, text_splitter=self._text_splitter
-        )
-        text_chunks = [text_split.text_chunk for text_split in text_splits]
-        doc_nodes = {
-            (start_idx + i): Node(
-                text=t,
-                index=(start_idx + i),
-                ref_doc_id=document.get_doc_id(),
-                embedding=document.embedding,
-                extra_info=document.extra_info,
-            )
-            for i, t in enumerate(text_chunks)
-        }
-        return doc_nodes
+    @property
+    def docstore(self) -> DocumentStore:
+        """Return docstore."""
+        return self._docstore
 
-    def build_from_text(
+    def build_from_nodes(
         self,
-        documents: Sequence[BaseDocument],
+        nodes: Sequence[Node],
         build_tree: bool = True,
     ) -> IndexGraph:
         """Build from text.
@@ -80,22 +57,25 @@ class GPTTreeIndexBuilder:
             IndexGraph: graph object consisting of all_nodes, root_nodes
 
         """
-        all_nodes: Dict[int, Node] = {}
-        for d in documents:
-            all_nodes.update(self._get_nodes_from_document(len(all_nodes), d))
+        index_graph = IndexGraph()
+        for node in nodes:
+            index_graph.insert(node)
 
         if build_tree:
-            # instantiate all_nodes from initial text chunks
-            root_nodes = self.build_index_from_nodes(all_nodes, all_nodes, level=0)
+            return self.build_index_from_nodes(
+                index_graph, index_graph.all_nodes, index_graph.all_nodes, level=0
+            )
         else:
-            # if build_tree is False, then don't surface any root nodes
-            root_nodes = {}
-        return IndexGraph(all_nodes=all_nodes, root_nodes=root_nodes)
+            return index_graph
 
     def _prepare_node_and_text_chunks(
-        self, cur_nodes: Dict[int, Node]
+        self, cur_node_ids: Dict[int, str]
     ) -> Tuple[List[int], List[List[Node]], List[str]]:
         """Prepare node and text chunks."""
+        cur_nodes = {
+            index: self._docstore.get_node(node_id)
+            for index, node_id in cur_node_ids.items()
+        }
         cur_node_list = get_sorted_node_list(cur_nodes)
         logger.info(
             f"> Building index from nodes: {len(cur_nodes) // self.num_children} chunks"
@@ -103,7 +83,7 @@ class GPTTreeIndexBuilder:
         indices, cur_nodes_chunks, text_chunks = [], [], []
         for i in range(0, len(cur_node_list), self.num_children):
             cur_nodes_chunk = cur_node_list[i : i + self.num_children]
-            text_chunk = self._prompt_helper.get_text_from_nodes(
+            text_chunk = self._service_context.prompt_helper.get_text_from_nodes(
                 cur_nodes_chunk, prompt=self.summary_prompt
             )
             indices.append(i)
@@ -113,12 +93,16 @@ class GPTTreeIndexBuilder:
 
     def _construct_parent_nodes(
         self,
-        cur_index: int,
+        index_graph: IndexGraph,
         indices: List[int],
         cur_nodes_chunks: List[List[Node]],
         summaries: List[str],
-    ) -> Dict[int, Node]:
-        """Construct parent nodes."""
+    ) -> Dict[int, str]:
+        """Construct parent nodes.
+
+        Save nodes to docstore.
+
+        """
         new_node_dict = {}
         for i, cur_nodes_chunk, new_summary in zip(
             indices, cur_nodes_chunks, summaries
@@ -129,28 +113,32 @@ class GPTTreeIndexBuilder:
             )
             new_node = Node(
                 text=new_summary,
-                index=cur_index,
-                child_indices={n.index for n in cur_nodes_chunk},
             )
-            new_node_dict[cur_index] = new_node
-            cur_index += 1
+            index_graph.insert(new_node, children_nodes=cur_nodes_chunk)
+            index = index_graph.get_index(new_node)
+            new_node_dict[index] = new_node.get_doc_id()
+            self._docstore.add_documents([new_node])
         return new_node_dict
 
     def build_index_from_nodes(
-        self, cur_nodes: Dict[int, Node], all_nodes: Dict[int, Node], level: int = 0
-    ) -> Dict[int, Node]:
+        self,
+        index_graph: IndexGraph,
+        cur_node_ids: Dict[int, str],
+        all_node_ids: Dict[int, str],
+        level: int = 0,
+    ) -> IndexGraph:
         """Consolidates chunks recursively, in a bottoms-up fashion."""
-        if len(cur_nodes) <= self.num_children:
-            return cur_nodes
+        if len(cur_node_ids) <= self.num_children:
+            index_graph.root_nodes = cur_node_ids
+            return index_graph
 
-        cur_index = len(all_nodes)
         indices, cur_nodes_chunks, text_chunks = self._prepare_node_and_text_chunks(
-            cur_nodes
+            cur_node_ids
         )
 
         if self._use_async:
             tasks = [
-                self._llm_predictor.apredict(
+                self._service_context.llm_predictor.apredict(
                     self.summary_prompt, context_str=text_chunk
                 )
                 for text_chunk in text_chunks
@@ -159,50 +147,67 @@ class GPTTreeIndexBuilder:
             summaries = [output[0] for output in outputs]
         else:
             summaries = [
-                self._llm_predictor.predict(
+                self._service_context.llm_predictor.predict(
                     self.summary_prompt, context_str=text_chunk
                 )[0]
                 for text_chunk in text_chunks
             ]
-        self._llama_logger.add_log({"summaries": summaries, "level": level})
+        self._service_context.llama_logger.add_log(
+            {"summaries": summaries, "level": level}
+        )
 
         new_node_dict = self._construct_parent_nodes(
-            cur_index, indices, cur_nodes_chunks, summaries
+            index_graph, indices, cur_nodes_chunks, summaries
         )
-        all_nodes.update(new_node_dict)
+        all_node_ids.update(new_node_dict)
+
+        index_graph.root_nodes = new_node_dict
 
         if len(new_node_dict) <= self.num_children:
-            return new_node_dict
+            return index_graph
         else:
             return self.build_index_from_nodes(
-                new_node_dict, all_nodes, level=level + 1
+                index_graph, new_node_dict, all_node_ids, level=level + 1
             )
 
     async def abuild_index_from_nodes(
-        self, cur_nodes: Dict[int, Node], all_nodes: Dict[int, Node], level: int = 0
-    ) -> Dict[int, Node]:
+        self,
+        index_graph: IndexGraph,
+        cur_node_ids: Dict[int, str],
+        all_node_ids: Dict[int, str],
+        level: int = 0,
+    ) -> IndexGraph:
         """Consolidates chunks recursively, in a bottoms-up fashion."""
-        cur_index = len(all_nodes)
+        if len(cur_node_ids) <= self.num_children:
+            index_graph.root_nodes = cur_node_ids
+            return index_graph
+
         indices, cur_nodes_chunks, text_chunks = self._prepare_node_and_text_chunks(
-            cur_nodes
+            cur_node_ids
         )
 
         tasks = [
-            self._llm_predictor.apredict(self.summary_prompt, context_str=text_chunk)
+            self._service_context.llm_predictor.apredict(
+                self.summary_prompt, context_str=text_chunk
+            )
             for text_chunk in text_chunks
         ]
         outputs: List[Tuple[str, str]] = await asyncio.gather(*tasks)
         summaries = [output[0] for output in outputs]
-        self._llama_logger.add_log({"summaries": summaries, "level": level})
+        self._service_context.llama_logger.add_log(
+            {"summaries": summaries, "level": level}
+        )
 
         new_node_dict = self._construct_parent_nodes(
-            cur_index, indices, cur_nodes_chunks, summaries
+            index_graph, indices, cur_nodes_chunks, summaries
         )
-        all_nodes.update(new_node_dict)
+        all_node_ids.update(new_node_dict)
+
+        index_graph.root_nodes = new_node_dict
 
         if len(new_node_dict) <= self.num_children:
-            return new_node_dict
+            return index_graph
         else:
             return await self.abuild_index_from_nodes(
-                new_node_dict, all_nodes, level=level + 1
+                index_graph, new_node_dict, all_node_ids, level=level + 1
             )
