@@ -4,7 +4,7 @@ from abc import abstractmethod
 from collections import defaultdict
 from typing import Any, Dict, List, Optional
 
-from gpt_index.data_structs.data_structs import KeywordTable, Node
+from gpt_index.data_structs.data_structs_v2 import KeywordTable, Node
 from gpt_index.indices.keyword_table.utils import (
     extract_keywords_given_response,
     rake_extract_keywords,
@@ -68,7 +68,7 @@ class BaseGPTKeywordTableQuery(BaseGPTIndexQuery[KeywordTable]):
     def _get_keywords(self, query_str: str) -> List[str]:
         """Extract keywords."""
 
-    def _get_nodes_for_response(
+    def _retrieve(
         self,
         query_bundle: QueryBundle,
         similarity_tracker: Optional[SimilarityTracker] = None,
@@ -79,21 +79,19 @@ class BaseGPTKeywordTableQuery(BaseGPTIndexQuery[KeywordTable]):
         logger.info(f"query keywords: {keywords}")
 
         # go through text chunks in order of most matching keywords
-        chunk_indices_count: Dict[int, int] = defaultdict(int)
+        chunk_indices_count: Dict[str, int] = defaultdict(int)
         keywords = [k for k in keywords if k in self.index_struct.keywords]
         logger.info(f"> Extracted keywords: {keywords}")
         for k in keywords:
-            for text_chunk_idx in self.index_struct.table[k]:
-                chunk_indices_count[text_chunk_idx] += 1
+            for node_id in self.index_struct.table[k]:
+                chunk_indices_count[node_id] += 1
         sorted_chunk_indices = sorted(
             list(chunk_indices_count.keys()),
             key=lambda x: chunk_indices_count[x],
             reverse=True,
         )
         sorted_chunk_indices = sorted_chunk_indices[: self.num_chunks_per_query]
-        sorted_nodes = [
-            self.index_struct.text_chunks[idx] for idx in sorted_chunk_indices
-        ]
+        sorted_nodes = self._docstore.get_nodes(sorted_chunk_indices)
         # filter sorted nodes
         for node_processor in self.node_preprocessors:
             sorted_nodes = node_processor.postprocess_nodes(sorted_nodes)
@@ -124,7 +122,7 @@ class GPTKeywordTableGPTQuery(BaseGPTKeywordTableQuery):
 
     def _get_keywords(self, query_str: str) -> List[str]:
         """Extract keywords."""
-        response, _ = self._llm_predictor.predict(
+        response, _ = self._service_context.llm_predictor.predict(
             self.query_keyword_extract_template,
             max_keywords=self.max_keywords_per_query,
             question=query_str,
