@@ -3,20 +3,18 @@
 from abc import abstractmethod
 from typing import Any, Callable, Dict, Generator, Optional, cast
 
-from gpt_index.data_structs.data_structs import IndexStruct
-from gpt_index.indices.prompt_helper import PromptHelper
-from gpt_index.indices.query.base import BaseGPTIndexQuery
+from gpt_index.data_structs.data_structs_v2 import V2IndexStruct as IndexStruct
 from gpt_index.indices.query.query_transform.base import (
     BaseQueryTransform,
     StepDecomposeQueryTransform,
 )
 from gpt_index.indices.query.schema import QueryBundle
 from gpt_index.indices.response.builder import ResponseBuilder, ResponseMode, TextChunk
-from gpt_index.langchain_helpers.chain_wrapper import LLMPredictor
+from gpt_index.indices.service_context import ServiceContext
 from gpt_index.prompts.default_prompt_selectors import DEFAULT_REFINE_PROMPT_SEL
 from gpt_index.prompts.default_prompts import DEFAULT_TEXT_QA_PROMPT
 from gpt_index.prompts.prompts import QuestionAnswerPrompt, RefinePrompt
-from gpt_index.response.schema import Response
+from gpt_index.response.schema import RESPONSE_TYPE, Response
 
 
 class BaseQueryCombiner:
@@ -26,20 +24,23 @@ class BaseQueryCombiner:
         self,
         index_struct: IndexStruct,
         query_transform: BaseQueryTransform,
+        query_runner: Any,  # NOTE: type as Any to avoid circular dependency
     ) -> None:
         """Init params."""
         self._index_struct = index_struct
         self._query_transform = query_transform
+        from gpt_index.indices.query.query_runner import QueryRunner
+
+        assert isinstance(query_runner, QueryRunner)
+        self._query_runner = query_runner
 
     @abstractmethod
-    def run(self, query_obj: BaseGPTIndexQuery, query_bundle: QueryBundle) -> Response:
+    def run(self, query_bundle: QueryBundle, level: int = 0) -> RESPONSE_TYPE:
         """Run query combiner."""
 
-    async def arun(
-        self, query_obj: BaseGPTIndexQuery, query_bundle: QueryBundle
-    ) -> Response:
+    async def arun(self, query_bundle: QueryBundle, level: int = 0) -> RESPONSE_TYPE:
         """Async run query combiner."""
-        return self.run(query_obj, query_bundle)
+        return self.run(query_bundle, level=level)
 
 
 class SingleQueryCombiner(BaseQueryCombiner):
@@ -59,17 +60,19 @@ class SingleQueryCombiner(BaseQueryCombiner):
         )
         return updated_query_bundle
 
-    def run(self, query_obj: BaseGPTIndexQuery, query_bundle: QueryBundle) -> Response:
+    def run(self, query_bundle: QueryBundle, level: int = 0) -> RESPONSE_TYPE:
         """Run query combiner."""
         updated_query_bundle = self._prepare_update(query_bundle)
-        return query_obj.query(updated_query_bundle)
+        return self._query_runner.query_transformed(
+            updated_query_bundle, self._index_struct, level=level
+        )
 
-    async def arun(
-        self, query_obj: BaseGPTIndexQuery, query_bundle: QueryBundle
-    ) -> Response:
+    async def arun(self, query_bundle: QueryBundle, level: int = 0) -> RESPONSE_TYPE:
         """Run query combiner."""
         updated_query_bundle = self._prepare_update(query_bundle)
-        return await query_obj.aquery(updated_query_bundle)
+        return await self._query_runner.aquery_transformed(
+            updated_query_bundle, self._index_struct, level=level
+        )
 
 
 def default_stop_fn(stop_dict: Dict) -> bool:
@@ -95,8 +98,8 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
         self,
         index_struct: IndexStruct,
         query_transform: BaseQueryTransform,
-        llm_predictor: Optional[LLMPredictor] = None,
-        prompt_helper: Optional[PromptHelper] = None,
+        query_runner: Any,
+        service_context: Optional[ServiceContext] = None,
         text_qa_template: Optional[QuestionAnswerPrompt] = None,
         refine_template: Optional[RefinePrompt] = None,
         response_mode: ResponseMode = ResponseMode.DEFAULT,
@@ -107,13 +110,16 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
         use_async: bool = True,
     ) -> None:
         """Init params."""
-        super().__init__(index_struct, query_transform=query_transform)
+        super().__init__(
+            index_struct, query_transform=query_transform, query_runner=query_runner
+        )
         self._index_struct = index_struct
         self._query_transform = query_transform
-        self._llm_predictor = llm_predictor or LLMPredictor()
-        self._prompt_helper = prompt_helper or PromptHelper.from_llm_predictor(
-            self._llm_predictor, chunk_size_limit=None
-        )
+        from gpt_index.indices.query.query_runner import QueryRunner
+
+        assert isinstance(query_runner, QueryRunner)
+        self._query_runner = query_runner
+        self._service_context = service_context or ServiceContext.from_defaults()
         self._num_steps = num_steps
         self._early_stopping = early_stopping
         # TODO: make interface to stop function better
@@ -126,8 +132,7 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
         self.text_qa_template = text_qa_template or DEFAULT_TEXT_QA_PROMPT
         self.refine_template = refine_template or DEFAULT_REFINE_PROMPT_SEL
         self.response_builder = ResponseBuilder(
-            self._prompt_helper,
-            self._llm_predictor,
+            self._service_context,
             self.text_qa_template,
             self.refine_template,
             use_async=use_async,
@@ -147,7 +152,7 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
         )
         return query_bundle
 
-    def run(self, query_obj: BaseGPTIndexQuery, query_bundle: QueryBundle) -> Response:
+    def run(self, query_bundle: QueryBundle, level: int = 0) -> RESPONSE_TYPE:
         """Run query combiner."""
         prev_reasoning = ""
         cur_response = None
@@ -173,12 +178,14 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
                 should_stop = True
                 break
 
-            cur_response = query_obj.query(updated_query_bundle)
+            cur_response = self._query_runner.query_transformed(
+                updated_query_bundle, self._index_struct, level=level
+            )
 
             # append to response builder
             cur_qa_text = (
                 f"\nQuestion: {updated_query_bundle.query_str}\n"
-                f"Answer: {cur_response.response}"
+                f"Answer: {str(cur_response)}"
             )
             self.response_builder.add_text_chunks([TextChunk(cur_qa_text)])
             for source_node in cur_response.source_nodes:
@@ -189,7 +196,7 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
             )
 
             prev_reasoning += (
-                f"- {updated_query_bundle.query_str}\n" f"- {cur_response.response}\n"
+                f"- {updated_query_bundle.query_str}\n" f"- {str(cur_response)}\n"
             )
             cur_steps += 1
 
@@ -211,6 +218,7 @@ class MultiStepQueryCombiner(BaseQueryCombiner):
 def get_default_query_combiner(
     index_struct: IndexStruct,
     query_transform: BaseQueryTransform,
+    query_runner: Any,  # NOTE: type as Any to avoid circular dependency
     extra_kwargs: Optional[Dict] = None,
 ) -> BaseQueryCombiner:
     """Get default query combiner."""
@@ -219,7 +227,10 @@ def get_default_query_combiner(
         return MultiStepQueryCombiner(
             index_struct,
             query_transform=query_transform,
-            llm_predictor=extra_kwargs.get("llm_predictor", None),
+            query_runner=query_runner,
+            service_context=extra_kwargs.get("service_context", None),
         )
     else:
-        return SingleQueryCombiner(index_struct, query_transform=query_transform)
+        return SingleQueryCombiner(
+            index_struct, query_transform=query_transform, query_runner=query_runner
+        )
