@@ -22,11 +22,15 @@ class EpubParser(BaseParser):
             import ebooklib
             from ebooklib import epub
         except ImportError:
-            raise ValueError("`EbookLib` is required to read Epub files.")
+            raise ImportError(
+                "`EbookLib` is required to read Epub files: `pip install EbookLib`"
+            )
         try:
             import html2text
         except ImportError:
-            raise ValueError("`html2text` is required to parse Epub files.")
+            raise ImportError(
+                "`html2text` is required to parse Epub files: `pip install html2text`"
+            )
 
         text_list = []
         book = epub.read_epub(file, options={"ignore_ncx": True})
