@@ -1,9 +1,9 @@
 """Test docstore."""
 
-from typing import Dict, Type
 
-from gpt_index.data_structs.data_structs import IndexStruct, Node
-from gpt_index.docstore import DocumentStore
+from gpt_index.constants import TYPE_KEY
+from gpt_index.data_structs.node_v2 import Node, NodeType
+from gpt_index.docstore_v2 import DocumentStore
 from gpt_index.readers.schema.base import Document
 
 
@@ -12,8 +12,6 @@ def test_docstore() -> None:
     doc = Document("hello world", doc_id="d1", extra_info={"foo": "bar"})
     node = Node("my node", doc_id="d2", node_info={"node": "info"})
 
-    type_to_struct: Dict[str, Type[IndexStruct]] = {"node": Node}
-
     # test get document
     docstore = DocumentStore.from_documents([doc, node])
     gd1 = docstore.get_document("d1")
@@ -28,7 +26,7 @@ def test_docstore() -> None:
         "doc_id": "d1",
         "embedding": None,
         "extra_info": {"foo": "bar"},
-        "__type__": "Document",
+        TYPE_KEY: "Document",
     }
     d2_expected: dict = {
         "text": "my node",
@@ -36,16 +34,13 @@ def test_docstore() -> None:
         "embedding": None,
         "extra_info": None,
         "node_info": {"node": "info"},
-        "index": 0,
-        "child_indices": [],
-        "ref_doc_id": None,
-        "image": None,
-        "__type__": "node",
+        "relationships": {},
+        TYPE_KEY: NodeType.TEXT,
     }
     doc_dict["docs"]["d1"].pop("doc_hash")
     doc_dict["docs"]["d2"].pop("doc_hash")
     assert doc_dict["docs"]["d1"] == d1_expected
     assert doc_dict["docs"]["d2"] == d2_expected
 
-    docstore_loaded = DocumentStore.load_from_dict(doc_dict, type_to_struct)
+    docstore_loaded = DocumentStore.load_from_dict(doc_dict)
     assert docstore_loaded == docstore
